@@ -1,5 +1,6 @@
 from __future__ import unicode_literals
 from django.http import HttpRequest
+from django.utils import six
 
 
 # In a separate file to avoid circular imports...
@@ -19,6 +20,7 @@ class Bundle(object):
                  related_name=None,
                  objects_saved=None,
                  related_objects_to_save=None,
+                 via_uri=False,
                  ):
         self.obj = obj
         self.data = data or {}
@@ -28,6 +30,10 @@ class Bundle(object):
         self.errors = {}
         self.objects_saved = objects_saved or set()
         self.related_objects_to_save = related_objects_to_save or {}
+        self.via_uri = via_uri
 
     def __repr__(self):
-        return "<Bundle for obj: '%s' and with data: '%s'>" % (self.obj, self.data)
+        repr_string = "<Bundle for obj: '%r' and with data: '%r'>"
+        if six.PY2:
+            repr_string = repr_string.encode('utf-8')
+        return repr_string % (self.obj, self.data)
