@@ -1,6 +1,7 @@
 """
 The various HTTP responses for use in returning proper HTTP codes.
 """
+from __future__ import unicode_literals
 from django.http import HttpResponse
 
 
@@ -21,6 +22,10 @@ class HttpAccepted(HttpResponse):
 class HttpNoContent(HttpResponse):
     status_code = 204
 
+    def __init__(self, *args, **kwargs):
+        super(HttpNoContent, self).__init__(*args, **kwargs)
+        del self['Content-Type']
+
 
 class HttpMultipleChoices(HttpResponse):
     status_code = 300
@@ -76,4 +81,3 @@ class HttpApplicationError(HttpResponse):
 
 class HttpNotImplemented(HttpResponse):
     status_code = 501
-
