@@ -9,7 +9,7 @@ class Command(NoArgsCommand):
     help = "Goes through all users and adds API keys for any that don't have one."
 
     def handle_noargs(self, **options):
-        """Goes through all users and adds API keys for any that don't have one."""
+        "Goes through all users and adds API keys for any that don't have one."
         self.verbosity = int(options.get('verbosity', 1))
 
         User = get_user_model()
