@@ -1,7 +1,9 @@
 from __future__ import unicode_literals
+
 import datetime
+import json
 import re
-import django
+
 from django.conf import settings
 from django.core.exceptions import ImproperlyConfigured
 from django.utils import six
@@ -10,19 +12,19 @@ from django.core.serializers import json as djangojson
 
 from tastypie.bundle import Bundle
 from tastypie.exceptions import BadRequest, UnsupportedFormat
-from tastypie.utils import format_datetime, format_date, format_time, make_naive
+from tastypie.utils import format_datetime, format_date, format_time,\
+    make_naive
 
 try:
     import defusedxml.lxml as lxml
     from defusedxml.common import DefusedXmlException
     from defusedxml.lxml import parse as parse_xml
-    from lxml.etree import Element, tostring, LxmlError, XMLParser
+    from lxml.etree import Element, tostring, LxmlError
 except ImportError:
     lxml = None
 
 try:
     import yaml
-    from django.core.serializers import pyyaml
 except ImportError:
     yaml = None
 
@@ -31,8 +33,6 @@ try:
 except ImportError:
     biplist = None
 
-import json
-
 
 XML_ENCODING = re.compile('<\?xml.*?\?>', re.IGNORECASE)
 
@@ -53,9 +53,13 @@ if yaml is not None:
             except UnicodeEncodeError:
                 return value
 
-    TastypieConstructor.add_constructor(u'tag:yaml.org,2002:python/unicode', TastypieConstructor.construct_yaml_unicode_dammit)
+    TastypieConstructor.add_constructor(
+        u'tag:yaml.org,2002:python/unicode',
+        TastypieConstructor.construct_yaml_unicode_dammit
+    )
 
-    class TastypieLoader(Reader, Scanner, Parser, Composer, TastypieConstructor, Resolver):
+    class TastypieLoader(Reader, Scanner, Parser, Composer,
+            TastypieConstructor, Resolver):
         def __init__(self, stream):
             Reader.__init__(self, stream)
             Scanner.__init__(self)
@@ -65,6 +69,34 @@ if yaml is not None:
             Resolver.__init__(self)
 
 
+_NUM = 0
+_DICT = 1
+_LIST = 2
+_STR = 3
+_BUNDLE = 4
+_DATETIME = 5
+_DATE = 6
+_TIME = 7
+
+_SIMPLETYPES = {
+    float: _NUM,
+    bool: _NUM,
+    dict: _DICT,
+    list: _LIST,
+    tuple: _LIST,
+    Bundle: _BUNDLE,
+    datetime.datetime: _DATETIME,
+    datetime.date: _DATE,
+    datetime.time: _TIME,
+}
+
+for integer_type in six.integer_types:
+    _SIMPLETYPES[integer_type] = _NUM
+
+for string_type in six.string_types:
+    _SIMPLETYPES[string_type] = _STR
+
+
 class Serializer(object):
     """
     A swappable class for serialization.
@@ -75,7 +107,6 @@ class Serializer(object):
         * jsonp (Disabled by default)
         * xml
         * yaml
-        * html
         * plist (see http://explorapp.com/biplist/)
 
     It was designed to make changing behavior easy, either by overridding the
@@ -83,20 +114,22 @@ class Serializer(object):
     ``formats/content_types`` options or by altering the other hook methods.
     """
 
-    formats = ['json', 'xml', 'yaml', 'html', 'plist']
+    formats = ['json', 'xml', 'yaml', 'plist']
 
-    content_types = {'json': 'application/json',
-                     'jsonp': 'text/javascript',
-                     'xml': 'application/xml',
-                     'yaml': 'text/yaml',
-                     'html': 'text/html',
-                     'plist': 'application/x-plist'}
+    content_types = {
+        'json': 'application/json',
+        'jsonp': 'text/javascript',
+        'xml': 'application/xml',
+        'yaml': 'text/yaml',
+        'plist': 'application/x-plist'
+    }
 
     def __init__(self, formats=None, content_types=None, datetime_formatting=None):
         if datetime_formatting is not None:
             self.datetime_formatting = datetime_formatting
         else:
-            self.datetime_formatting = getattr(settings, 'TASTYPIE_DATETIME_FORMATTING', 'iso-8601')
+            self.datetime_formatting = getattr(settings,
+                'TASTYPIE_DATETIME_FORMATTING', 'iso-8601')
 
         self.supported_formats = []
 
@@ -107,12 +140,13 @@ class Serializer(object):
             self.formats = formats
 
         if self.formats is Serializer.formats and hasattr(settings, 'TASTYPIE_DEFAULT_FORMATS'):
-            # We want TASTYPIE_DEFAULT_FORMATS to override unmodified defaults but not intentational changes
-            # on Serializer subclasses:
+            # We want TASTYPIE_DEFAULT_FORMATS to override unmodified defaults
+            # but not intentational changes on Serializer subclasses:
             self.formats = settings.TASTYPIE_DEFAULT_FORMATS
 
         if not isinstance(self.formats, (list, tuple)):
-            raise ImproperlyConfigured('Formats should be a list or tuple, not %r' % self.formats)
+            raise ImproperlyConfigured(
+                'Formats should be a list or tuple, not %r' % self.formats)
 
         for format in self.formats:
             try:
@@ -120,6 +154,24 @@ class Serializer(object):
             except KeyError:
                 raise ImproperlyConfigured("Content type for specified type '%s' not found. Please provide it at either the class level or via the arguments." % format)
 
+        # Reverse the list, because mimeparse is weird like that. See also
+        # https://github.com/django-tastypie/django-tastypie/issues#issue/12 for
+        # more information.
+        self.supported_formats_reversed = list(self.supported_formats)
+        self.supported_formats_reversed.reverse()
+
+        self._from_methods = {}
+        self._to_methods = {}
+
+        for short_format, long_format in self.content_types.items():
+            method = getattr(self, "from_%s" % short_format, None)
+
+            self._from_methods[long_format] = method
+
+            method = getattr(self, "to_%s" % short_format, None)
+
+            self._to_methods[long_format] = method
+
     def get_mime_for_format(self, format):
         """
         Given a format, attempts to determine the correct MIME type.
@@ -146,7 +198,7 @@ class Serializer(object):
             return format_datetime(data)
         if self.datetime_formatting == 'iso-8601-strict':
             # Remove microseconds to strictly adhere to iso-8601
-            data = data - datetime.timedelta(microseconds = data.microsecond)
+            data = data - datetime.timedelta(microseconds=data.microsecond)
 
         return data.isoformat()
 
@@ -177,7 +229,10 @@ class Serializer(object):
             return format_time(data)
         if self.datetime_formatting == 'iso-8601-strict':
             # Remove microseconds to strictly adhere to iso-8601
-            data = (datetime.datetime.combine(datetime.date(1,1,1),data) - datetime.timedelta(microseconds = data.microsecond)).time()
+            data = (
+                datetime.datetime.combine(datetime.date(1, 1, 1), data) -
+                datetime.timedelta(microseconds=data.microsecond)
+            ).time()
 
         return data.isoformat()
 
@@ -186,45 +241,35 @@ class Serializer(object):
         Given some data and a format, calls the correct method to serialize
         the data and returns the result.
         """
-        desired_format = None
+        method = None
         if options is None:
             options = {}
 
-        for short_format, long_format in self.content_types.items():
-            if format == long_format:
-                if hasattr(self, "to_%s" % short_format):
-                    desired_format = short_format
-                    break
+        method = self._to_methods.get(format)
 
-        if desired_format is None:
+        if method is None:
             raise UnsupportedFormat("The format indicated '%s' had no available serialization method. Please check your ``formats`` and ``content_types`` on your Serializer." % format)
 
-        serialized = getattr(self, "to_%s" % desired_format)(bundle, options)
-        return serialized
+        return method(bundle, options)
 
     def deserialize(self, content, format='application/json'):
         """
         Given some data and a format, calls the correct method to deserialize
         the data and returns the result.
         """
-        desired_format = None
+        method = None
 
         format = format.split(';')[0]
 
-        for short_format, long_format in self.content_types.items():
-            if format == long_format:
-                if hasattr(self, "from_%s" % short_format):
-                    desired_format = short_format
-                    break
+        method = self._from_methods.get(format)
 
-        if desired_format is None:
+        if method is None:
             raise UnsupportedFormat("The format indicated '%s' had no available deserialization method. Please check your ``formats`` and ``content_types`` on your Serializer." % format)
 
         if isinstance(content, six.binary_type):
             content = force_text(content)
 
-        deserialized = getattr(self, "from_%s" % desired_format)(content)
-        return deserialized
+        return method(content)
 
     def to_simple(self, data, options):
         """
@@ -234,39 +279,39 @@ class Serializer(object):
         This brings complex Python data structures down to native types of the
         serialization format(s).
         """
-        if isinstance(data, (list, tuple)):
-            return [self.to_simple(item, options) for item in data]
-        if isinstance(data, dict):
-            return dict((key, self.to_simple(val, options)) for (key, val) in data.items())
-        elif isinstance(data, Bundle):
-            return dict((key, self.to_simple(val, options)) for (key, val) in data.data.items())
-        elif hasattr(data, 'dehydrated_type'):
-            if getattr(data, 'dehydrated_type', None) == 'related' and data.is_m2m == False:
-                if data.full:
-                    return self.to_simple(data.fk_resource, options)
-                else:
-                    return self.to_simple(data.value, options)
-            elif getattr(data, 'dehydrated_type', None) == 'related' and data.is_m2m == True:
-                if data.full:
-                    return [self.to_simple(bundle, options) for bundle in data.m2m_bundles]
-                else:
-                    return [self.to_simple(val, options) for val in data.value]
-            else:
-                return self.to_simple(data.value, options)
-        elif isinstance(data, datetime.datetime):
-            return self.format_datetime(data)
-        elif isinstance(data, datetime.date):
-            return self.format_date(data)
-        elif isinstance(data, datetime.time):
-            return self.format_time(data)
-        elif isinstance(data, bool):
-            return data
-        elif isinstance(data, (six.integer_types, float)):
-            return data
-        elif data is None:
+        if data is None:
             return None
-        else:
+
+        data_type = type(data)
+
+        stype = _STR
+
+        for dt in data_type.__mro__:
+            try:
+                stype = _SIMPLETYPES[dt]
+                break
+            except KeyError:
+                pass
+
+        if stype == _NUM:
+            return data
+        if stype == _DICT:
+            to_simple = self.to_simple
+            return {key: to_simple(val, options) for key, val in six.iteritems(data)}
+        if stype == _STR:
             return force_text(data)
+        if stype == _LIST:
+            to_simple = self.to_simple
+            return [to_simple(item, options) for item in data]
+        if stype == _BUNDLE:
+            to_simple = self.to_simple
+            return {key: to_simple(val, options) for key, val in six.iteritems(data.data)}
+        if stype == _DATETIME:
+            return self.format_datetime(data)
+        if stype == _DATE:
+            return self.format_date(data)
+        if stype == _TIME:
+            return self.format_time(data)
 
     def to_etree(self, data, options=None, name=None, depth=0):
         """
@@ -281,7 +326,7 @@ class Serializer(object):
             else:
                 element = Element('objects')
             for item in data:
-                element.append(self.to_etree(item, options, depth=depth+1))
+                element.append(self.to_etree(item, options, depth=depth + 1))
                 element[:] = sorted(element, key=lambda x: x.tag)
         elif isinstance(data, dict):
             if depth == 0:
@@ -290,30 +335,15 @@ class Serializer(object):
                 element = Element(name or 'object')
                 element.set('type', 'hash')
             for (key, value) in data.items():
-                element.append(self.to_etree(value, options, name=key, depth=depth+1))
+                element.append(self.to_etree(
+                    value, options, name=key, depth=depth + 1))
                 element[:] = sorted(element, key=lambda x: x.tag)
         elif isinstance(data, Bundle):
             element = Element(name or 'object')
             for field_name, field_object in data.data.items():
-                element.append(self.to_etree(field_object, options, name=field_name, depth=depth+1))
+                element.append(self.to_etree(
+                    field_object, options, name=field_name, depth=depth + 1))
                 element[:] = sorted(element, key=lambda x: x.tag)
-        elif hasattr(data, 'dehydrated_type'):
-            if getattr(data, 'dehydrated_type', None) == 'related' and data.is_m2m == False:
-                if data.full:
-                    return self.to_etree(data.fk_resource, options, name, depth+1)
-                else:
-                    return self.to_etree(data.value, options, name, depth+1)
-            elif getattr(data, 'dehydrated_type', None) == 'related' and data.is_m2m == True:
-                if data.full:
-                    element = Element(name or 'objects')
-                    for bundle in data.m2m_bundles:
-                        element.append(self.to_etree(bundle, options, bundle.resource_name, depth+1))
-                else:
-                    element = Element(name or 'objects')
-                    for value in data.value:
-                        element.append(self.to_etree(value, options, name, depth=depth+1))
-            else:
-                return self.to_etree(data.value, options, name)
         else:
             element = Element(name or 'value')
             simple_data = self.to_simple(data, options)
@@ -343,9 +373,15 @@ class Serializer(object):
             for element in elements:
                 if element.tag in ('object', 'objects'):
                     return self.from_etree(element)
-            return dict((element.tag, self.from_etree(element)) for element in elements)
+            return {
+                element.tag: self.from_etree(element)
+                for element in elements
+            }
         elif data.tag == 'object' or data.get('type') == 'hash':
-            return dict((element.tag, self.from_etree(element)) for element in data.getchildren())
+            return {
+                element.tag: self.from_etree(element)
+                for element in data.getchildren()
+            }
         elif data.tag == 'objects' or data.get('type') == 'list':
             return [self.from_etree(element) for element in data.getchildren()]
         else:
@@ -371,7 +407,8 @@ class Serializer(object):
         options = options or {}
         data = self.to_simple(data, options)
 
-        return djangojson.json.dumps(data, cls=djangojson.DjangoJSONEncoder, sort_keys=True, ensure_ascii=False)
+        return djangojson.json.dumps(data, cls=djangojson.DjangoJSONEncoder,
+            sort_keys=True, ensure_ascii=False)
 
     def from_json(self, content):
         """
@@ -380,7 +417,7 @@ class Serializer(object):
         try:
             return json.loads(content)
         except ValueError:
-            raise BadRequest
+            raise BadRequest('Request is not valid JSON.')
 
     def to_jsonp(self, data, options=None):
         """
@@ -393,9 +430,9 @@ class Serializer(object):
         details.
         """
         options = options or {}
-        json = self.to_json(data, options)
-        json = json.replace(u'\u2028', u'\\u2028').replace(u'\u2029', u'\\u2029')
-        return u'%s(%s)' % (options['callback'], json)
+        jsonstr = self.to_json(data, options).replace(
+            u'\u2028', u'\\u2028').replace(u'\u2029', u'\\u2029')
+        return u'%s(%s)' % (options['callback'], jsonstr)
 
     def to_xml(self, data, options=None):
         """
@@ -404,9 +441,11 @@ class Serializer(object):
         options = options or {}
 
         if lxml is None:
-            raise ImproperlyConfigured("Usage of the XML aspects requires lxml and defusedxml.")
+            raise ImproperlyConfigured(
+                "Usage of the XML aspects requires lxml and defusedxml.")
 
-        return tostring(self.to_etree(data, options), xml_declaration=True, encoding='utf-8')
+        return tostring(self.to_etree(data, options), xml_declaration=True,
+            encoding='utf-8')
 
     def from_xml(self, content, forbid_dtd=True, forbid_entities=True):
         """
@@ -417,7 +456,8 @@ class Serializer(object):
         necessary.
         """
         if lxml is None:
-            raise ImproperlyConfigured("Usage of the XML aspects requires lxml and defusedxml.")
+            raise ImproperlyConfigured(
+                "Usage of the XML aspects requires lxml and defusedxml.")
 
         try:
             # Stripping the encoding declaration. Because lxml.
@@ -440,7 +480,8 @@ class Serializer(object):
         options = options or {}
 
         if yaml is None:
-            raise ImproperlyConfigured("Usage of the YAML aspects requires yaml.")
+            raise ImproperlyConfigured(
+                "Usage of the YAML aspects requires yaml.")
 
         return yaml.dump(self.to_simple(data, options))
 
@@ -449,7 +490,8 @@ class Serializer(object):
         Given some YAML data, returns a Python dictionary of the decoded data.
         """
         if yaml is None:
-            raise ImproperlyConfigured("Usage of the YAML aspects requires yaml.")
+            raise ImproperlyConfigured(
+                "Usage of the YAML aspects requires yaml.")
 
         return yaml.load(content, Loader=TastypieLoader)
 
@@ -460,42 +502,25 @@ class Serializer(object):
         options = options or {}
 
         if biplist is None:
-            raise ImproperlyConfigured("Usage of the plist aspects requires biplist.")
+            raise ImproperlyConfigured(
+                "Usage of the plist aspects requires biplist.")
 
         return biplist.writePlistToString(self.to_simple(data, options))
 
     def from_plist(self, content):
         """
-        Given some binary plist data, returns a Python dictionary of the decoded data.
+        Given some binary plist data, returns a Python dictionary of the
+        decoded data.
         """
         if biplist is None:
-            raise ImproperlyConfigured("Usage of the plist aspects requires biplist.")
+            raise ImproperlyConfigured(
+                "Usage of the plist aspects requires biplist.")
 
         if isinstance(content, six.text_type):
             content = smart_bytes(content)
 
         return biplist.readPlistFromString(content)
 
-    def to_html(self, data, options=None):
-        """
-        Reserved for future usage.
-
-        The desire is to provide HTML output of a resource, making an API
-        available to a browser. This is on the TODO list but not currently
-        implemented.
-        """
-        options = options or {}
-        return 'Sorry, not implemented yet. Please append "?format=json" to your URL.'
-
-    def from_html(self, content):
-        """
-        Reserved for future usage.
-
-        The desire is to handle form-based (maybe Javascript?) input, making an
-        API available to a browser. This is on the TODO list but not currently
-        implemented.
-        """
-        pass
 
 def get_type_string(data):
     """
