@@ -1,18 +1,14 @@
 from __future__ import unicode_literals
 import time
+import warnings
 
 from django.conf import settings
 from django.test import TestCase
-from django.test.client import FakePayload, Client
+from django.test.client import Client
 from django.utils.encoding import force_text
 
 from tastypie.serializers import Serializer
 
-try:
-    from urllib.parse import urlparse
-except ImportError:
-    from urlparse import urlparse
-
 
 class TestApiClient(object):
     def __init__(self, serializer=None):
@@ -30,29 +26,37 @@ class TestApiClient(object):
 
     def get_content_type(self, short_format):
         """
-        Given a short name (such as ``json`` or ``xml``), returns the full content-type
-        for it (``application/json`` or ``application/xml`` in this case).
+        Given a short name (such as ``json`` or ``xml``), returns the full
+        content-type for it (``application/json`` or ``application/xml`` in
+        this case).
         """
         return self.serializer.content_types.get(short_format, 'json')
 
-    def get(self, uri, format='json', data=None, authentication=None, **kwargs):
+    def get(self, uri, format='json', data=None, authentication=None,
+            **kwargs):
         """
         Performs a simulated ``GET`` request to the provided URI.
 
-        Optionally accepts a ``data`` kwarg, which in the case of ``GET``, lets you
-        send along ``GET`` parameters. This is useful when testing filtering or other
-        things that read off the ``GET`` params. Example::
+        Optionally accepts a ``data`` kwarg, which in the case of ``GET``, lets
+        you send along ``GET`` parameters. This is useful when testing
+        filtering or other things that read off the ``GET`` params. Example::
 
             from tastypie.test import TestApiClient
             client = TestApiClient()
 
-            response = client.get('/api/v1/entry/1/', data={'format': 'json', 'title__startswith': 'a', 'limit': 20, 'offset': 60})
+            response = client.get('/api/v1/entry/1/', data={
+                'format': 'json',
+                'title__startswith': 'a',
+                'limit': 20,
+                'offset': 60
+            })
 
-        Optionally accepts an ``authentication`` kwarg, which should be an HTTP header
-        with the correct authentication data already setup.
+        Optionally accepts an ``authentication`` kwarg, which should be an HTTP
+        header with the correct authentication data already setup.
 
         All other ``**kwargs`` passed in get passed through to the Django
-        ``TestClient``. See https://docs.djangoproject.com/en/dev/topics/testing/#module-django.test.client
+        ``TestClient``. See
+https://docs.djangoproject.com/en/dev/topics/testing/#module-django.test.client
         for details.
         """
         content_type = self.get_content_type(format)
@@ -67,12 +71,14 @@ class TestApiClient(object):
 
         return self.client.get(uri, **kwargs)
 
-    def post(self, uri, format='json', data=None, authentication=None, **kwargs):
+    def post(self, uri, format='json', data=None, authentication=None,
+            **kwargs):
         """
         Performs a simulated ``POST`` request to the provided URI.
 
-        Optionally accepts a ``data`` kwarg. **Unlike** ``GET``, in ``POST`` the
-        ``data`` gets serialized & sent as the body instead of becoming part of the URI.
+        Optionally accepts a ``data`` kwarg. **Unlike** ``GET``, in ``POST``
+        the ``data`` gets serialized & sent as the body instead of becoming
+        part of the URI.
         Example::
 
             from tastypie.test import TestApiClient
@@ -85,30 +91,34 @@ class TestApiClient(object):
                 'user': '/api/v1/user/1/',
             })
 
-        Optionally accepts an ``authentication`` kwarg, which should be an HTTP header
-        with the correct authentication data already setup.
+        Optionally accepts an ``authentication`` kwarg, which should be an HTTP
+        header with the correct authentication data already setup.
 
         All other ``**kwargs`` passed in get passed through to the Django
-        ``TestClient``. See https://docs.djangoproject.com/en/dev/topics/testing/#module-django.test.client
+        ``TestClient``. See
+https://docs.djangoproject.com/en/dev/topics/testing/#module-django.test.client
         for details.
         """
         content_type = self.get_content_type(format)
         kwargs['content_type'] = content_type
 
         if data is not None:
-            kwargs['data'] = self.serializer.serialize(data, format=content_type)
+            kwargs['data'] = self.serializer.serialize(
+                data, format=content_type)
 
         if authentication is not None:
             kwargs['HTTP_AUTHORIZATION'] = authentication
 
         return self.client.post(uri, **kwargs)
 
-    def put(self, uri, format='json', data=None, authentication=None, **kwargs):
+    def put(self, uri, format='json', data=None, authentication=None,
+            **kwargs):
         """
         Performs a simulated ``PUT`` request to the provided URI.
 
         Optionally accepts a ``data`` kwarg. **Unlike** ``GET``, in ``PUT`` the
-        ``data`` gets serialized & sent as the body instead of becoming part of the URI.
+        ``data`` gets serialized & sent as the body instead of becoming part of
+        the URI.
         Example::
 
             from tastypie.test import TestApiClient
@@ -121,30 +131,34 @@ class TestApiClient(object):
                 'user': '/api/v1/user/1/',
             })
 
-        Optionally accepts an ``authentication`` kwarg, which should be an HTTP header
-        with the correct authentication data already setup.
+        Optionally accepts an ``authentication`` kwarg, which should be an HTTP
+        header with the correct authentication data already setup.
 
         All other ``**kwargs`` passed in get passed through to the Django
-        ``TestClient``. See https://docs.djangoproject.com/en/dev/topics/testing/#module-django.test.client
+        ``TestClient``. See
+https://docs.djangoproject.com/en/dev/topics/testing/#module-django.test.client
         for details.
         """
         content_type = self.get_content_type(format)
         kwargs['content_type'] = content_type
 
         if data is not None:
-            kwargs['data'] = self.serializer.serialize(data, format=content_type)
+            kwargs['data'] = self.serializer.serialize(
+                data, format=content_type)
 
         if authentication is not None:
             kwargs['HTTP_AUTHORIZATION'] = authentication
 
         return self.client.put(uri, **kwargs)
 
-    def patch(self, uri, format='json', data=None, authentication=None, **kwargs):
+    def patch(self, uri, format='json', data=None, authentication=None,
+            **kwargs):
         """
         Performs a simulated ``PATCH`` request to the provided URI.
 
-        Optionally accepts a ``data`` kwarg. **Unlike** ``GET``, in ``PATCH`` the
-        ``data`` gets serialized & sent as the body instead of becoming part of the URI.
+        Optionally accepts a ``data`` kwarg. **Unlike** ``GET``, in ``PATCH``
+        the ``data`` gets serialized & sent as the body instead of becoming
+        part of the URI.
         Example::
 
             from tastypie.test import TestApiClient
@@ -157,53 +171,48 @@ class TestApiClient(object):
                 'user': '/api/v1/user/1/',
             })
 
-        Optionally accepts an ``authentication`` kwarg, which should be an HTTP header
-        with the correct authentication data already setup.
+        Optionally accepts an ``authentication`` kwarg, which should be an HTTP
+        header with the correct authentication data already setup.
 
         All other ``**kwargs`` passed in get passed through to the Django
-        ``TestClient``. See https://docs.djangoproject.com/en/dev/topics/testing/#module-django.test.client
+        ``TestClient``. See
+https://docs.djangoproject.com/en/dev/topics/testing/#module-django.test.client
         for details.
         """
         content_type = self.get_content_type(format)
         kwargs['content_type'] = content_type
 
         if data is not None:
-            kwargs['data'] = self.serializer.serialize(data, format=content_type)
+            kwargs['data'] = self.serializer.serialize(
+                data, format=content_type)
 
         if authentication is not None:
             kwargs['HTTP_AUTHORIZATION'] = authentication
 
-        # This hurts because Django doesn't support PATCH natively.
-        parsed = urlparse(uri)
-        r = {
-            'CONTENT_LENGTH': len(kwargs['data']),
-            'CONTENT_TYPE': content_type,
-            'PATH_INFO': self.client._get_path(parsed),
-            'QUERY_STRING': parsed[4],
-            'REQUEST_METHOD': 'PATCH',
-            'wsgi.input': FakePayload(kwargs['data']),
-        }
-        r.update(kwargs)
-        return self.client.request(**r)
+        return self.client.patch(uri, **kwargs)
 
-    def delete(self, uri, format='json', data=None, authentication=None, **kwargs):
+    def delete(self, uri, format='json', data=None, authentication=None,
+            **kwargs):
         """
         Performs a simulated ``DELETE`` request to the provided URI.
 
-        Optionally accepts a ``data`` kwarg, which in the case of ``DELETE``, lets you
-        send along ``DELETE`` parameters. This is useful when testing filtering or other
-        things that read off the ``DELETE`` params. Example::
+        Optionally accepts a ``data`` kwarg, which in the case of ``DELETE``,
+        lets you send along ``DELETE`` parameters. This is useful when testing
+        filtering or other things that read off the ``DELETE`` params.
+        Example::
 
             from tastypie.test import TestApiClient
             client = TestApiClient()
 
-            response = client.delete('/api/v1/entry/1/', data={'format': 'json'})
+            response = client.delete('/api/v1/entry/1/',
+                data={'format': 'json'})
 
-        Optionally accepts an ``authentication`` kwarg, which should be an HTTP header
-        with the correct authentication data already setup.
+        Optionally accepts an ``authentication`` kwarg, which should be an HTTP
+        header with the correct authentication data already setup.
 
         All other ``**kwargs`` passed in get passed through to the Django
-        ``TestClient``. See https://docs.djangoproject.com/en/dev/topics/testing/#module-django.test.client
+        ``TestClient``. See
+https://docs.djangoproject.com/en/dev/topics/testing/#module-django.test.client
         for details.
         """
         content_type = self.get_content_type(format)
@@ -219,12 +228,13 @@ class TestApiClient(object):
         return self.client.delete(uri, **kwargs)
 
 
-class ResourceTestCase(TestCase):
+class ResourceTestCaseMixin(object):
     """
-    A useful base class for the start of testing Tastypie APIs.
+    A mixin of useful methods for testing Tastypie APIs.
+    Below we use this to subclass Django's TestCase and TransactionTestCase classes.
     """
     def setUp(self):
-        super(ResourceTestCase, self).setUp()
+        super(ResourceTestCaseMixin, self).setUp()
         self.serializer = Serializer()
         self.api_client = TestApiClient()
 
@@ -244,7 +254,8 @@ class ResourceTestCase(TestCase):
                 # Then the usual tests...
 
         """
-        raise NotImplementedError("You must return the class for your Resource to test.")
+        raise NotImplementedError(
+            "You must return the class for your Resource to test.")
 
     def create_basic(self, username, password):
         """
@@ -252,7 +263,8 @@ class ResourceTestCase(TestCase):
         Auth.
         """
         import base64
-        return 'Basic %s' % base64.b64encode(':'.join([username, password]).encode('utf-8')).decode('utf-8')
+        return 'Basic %s' % base64.b64encode(
+            ':'.join([username, password]).encode('utf-8')).decode('utf-8')
 
     def create_apikey(self, username, api_key):
         """
@@ -269,13 +281,21 @@ class ResourceTestCase(TestCase):
         from tastypie.authentication import hmac, sha1, uuid, python_digest
 
         new_uuid = uuid.uuid4()
-        opaque = hmac.new(str(new_uuid).encode('utf-8'), digestmod=sha1).hexdigest().decode('utf-8')
+        opaque = hmac.new(
+            str(new_uuid).encode('utf-8'), digestmod=sha1
+        ).hexdigest().decode('utf-8')
         return python_digest.build_authorization_request(
             username,
             method.upper(),
             uri,
-            1, # nonce_count
-            digest_challenge=python_digest.build_digest_challenge(time.time(), getattr(settings, 'SECRET_KEY', ''), 'django-tastypie', opaque, False),
+            1,  # nonce_count
+            digest_challenge=python_digest.build_digest_challenge(
+                time.time(),
+                settings.SECRET_KEY,
+                'django-tastypie',
+                opaque,
+                False
+            ),
             password=api_key
         )
 
@@ -293,12 +313,16 @@ class ResourceTestCase(TestCase):
             'name': 'Test',
             'description': 'Testing...'
         })
-        token, _ = Token.objects.get_or_create(key='foo', token_type=Token.ACCESS, defaults={
-            'consumer': consumer,
-            'resource': resource,
-            'secret': '',
-            'user': user,
-        })
+        token, _ = Token.objects.get_or_create(
+            key='foo',
+            token_type=Token.ACCESS,
+            defaults={
+                'consumer': consumer,
+                'resource': resource,
+                'secret': '',
+                'user': user,
+            }
+        )
 
         # Then generate the header.
         oauth_data = {
@@ -309,7 +333,9 @@ class ResourceTestCase(TestCase):
             'oauth_timestamp': str(int(time.time())),
             'oauth_token': 'foo',
         }
-        return 'OAuth %s' % ','.join([key+'='+value for key, value in oauth_data.items()])
+        return 'OAuth %s' % ','.join([
+            key + '=' + value for key, value in oauth_data.items()
+        ])
 
     def assertHttpOK(self, resp):
         """
@@ -327,7 +353,7 @@ class ResourceTestCase(TestCase):
         """
         Ensures the response is returning either a HTTP 202 or a HTTP 204.
         """
-        return self.assertIn(resp.status_code, [202, 204])
+        self.assertIn(resp.status_code, [202, 204])
 
     def assertHttpMultipleChoices(self, resp):
         """
@@ -415,10 +441,11 @@ class ResourceTestCase(TestCase):
 
     def assertValidJSON(self, data):
         """
-        Given the provided ``data`` as a string, ensures that it is valid JSON &
-        can be loaded properly.
+        Given the provided ``data`` as a string, ensures that it is valid JSON
+        & can be loaded properly.
         """
-        # Just try the load. If it throws an exception, the test case will fail.
+        # Just try the load. If it throws an exception, the test case will
+        # fail.
         self.serializer.from_json(data)
 
     def assertValidXML(self, data):
@@ -426,15 +453,17 @@ class ResourceTestCase(TestCase):
         Given the provided ``data`` as a string, ensures that it is valid XML &
         can be loaded properly.
         """
-        # Just try the load. If it throws an exception, the test case will fail.
+        # Just try the load. If it throws an exception, the test case will
+        # fail.
         self.serializer.from_xml(data)
 
     def assertValidYAML(self, data):
         """
-        Given the provided ``data`` as a string, ensures that it is valid YAML &
-        can be loaded properly.
+        Given the provided ``data`` as a string, ensures that it is valid YAML
+        & can be loaded properly.
         """
-        # Just try the load. If it throws an exception, the test case will fail.
+        # Just try the load. If it throws an exception, the test case will
+        # fail.
         self.serializer.from_yaml(data)
 
     def assertValidPlist(self, data):
@@ -442,13 +471,14 @@ class ResourceTestCase(TestCase):
         Given the provided ``data`` as a string, ensures that it is valid
         binary plist & can be loaded properly.
         """
-        # Just try the load. If it throws an exception, the test case will fail.
+        # Just try the load. If it throws an exception, the test case will
+        # fail.
         self.serializer.from_plist(data)
 
     def assertValidJSONResponse(self, resp):
         """
-        Given a ``HttpResponse`` coming back from using the ``client``, assert that
-        you get back:
+        Given a ``HttpResponse`` coming back from using the ``client``, assert
+        that you get back:
 
         * An HTTP 200
         * The correct content-type (``application/json``)
@@ -460,8 +490,8 @@ class ResourceTestCase(TestCase):
 
     def assertValidXMLResponse(self, resp):
         """
-        Given a ``HttpResponse`` coming back from using the ``client``, assert that
-        you get back:
+        Given a ``HttpResponse`` coming back from using the ``client``, assert
+        that you get back:
 
         * An HTTP 200
         * The correct content-type (``application/xml``)
@@ -473,8 +503,8 @@ class ResourceTestCase(TestCase):
 
     def assertValidYAMLResponse(self, resp):
         """
-        Given a ``HttpResponse`` coming back from using the ``client``, assert that
-        you get back:
+        Given a ``HttpResponse`` coming back from using the ``client``, assert
+        that you get back:
 
         * An HTTP 200
         * The correct content-type (``text/yaml``)
@@ -486,8 +516,8 @@ class ResourceTestCase(TestCase):
 
     def assertValidPlistResponse(self, resp):
         """
-        Given a ``HttpResponse`` coming back from using the ``client``, assert that
-        you get back:
+        Given a ``HttpResponse`` coming back from using the ``client``, assert
+        that you get back:
 
         * An HTTP 200
         * The correct content-type (``application/x-plist``)
@@ -499,28 +529,44 @@ class ResourceTestCase(TestCase):
 
     def deserialize(self, resp):
         """
-        Given a ``HttpResponse`` coming back from using the ``client``, this method
-        checks the ``Content-Type`` header & attempts to deserialize the data based on
-        that.
+        Given a ``HttpResponse`` coming back from using the ``client``, this
+        method checks the ``Content-Type`` header & attempts to deserialize the
+        data based on that.
 
-        It returns a Python datastructure (typically a ``dict``) of the serialized data.
+        It returns a Python datastructure (typically a ``dict``) of the
+        serialized data.
         """
-        return self.serializer.deserialize(resp.content, format=resp['Content-Type'])
+        return self.serializer.deserialize(
+            resp.content, format=resp['Content-Type'])
 
     def serialize(self, data, format='application/json'):
         """
-        Given a Python datastructure (typically a ``dict``) & a desired content-type,
-        this method will return a serialized string of that data.
+        Given a Python datastructure (typically a ``dict``) & a desired
+        content-type, this method will return a serialized string of that data.
         """
         return self.serializer.serialize(data, format=format)
 
     def assertKeys(self, data, expected):
         """
-        This method ensures that the keys of the ``data`` match up to the keys of
-        ``expected``.
+        This method ensures that the keys of the ``data`` match up to the keys
+        of ``expected``.
 
-        It covers the (extremely) common case where you want to make sure the keys of
-        a response match up to what is expected. This is typically less fragile than
-        testing the full structure, which can be prone to data changes.
+        It covers the (extremely) common case where you want to make sure the
+        keys of a response match up to what is expected. This is typically less
+        fragile than testing the full structure, which can be prone to data
+        changes.
         """
         self.assertEqual(sorted(data.keys()), sorted(expected))
+
+
+class ResourceTestCase(ResourceTestCaseMixin, TestCase):
+    """
+    This class exists for backwards compatibility, from before ResourceTestCaseMixin was added.
+    It throws a deprecation warning.
+    """
+    def __init__(self, *args, **kwargs):
+        warnings.warn(
+            "'ResourceTestCase' is deprecated & will be removed by v1.0.0. "
+            "Please use ``ResourceTestCaseMixin`` instead. "
+            "For example: ``class MyTest(ResourceTestCaseMixin, django.test.TestCase):``.")
+        super(ResourceTestCase, self).__init__(*args, **kwargs)
