@@ -1,18 +1,23 @@
 import datetime
-from dateutil.tz import *
+from dateutil.tz import tzoffset
+from decimal import Decimal
+
 from django.db import models
 from django.contrib.auth.models import User
 from django.test import TestCase
 from django.http import HttpRequest
+
 from tastypie.bundle import Bundle
 from tastypie.exceptions import ApiFieldError, NotFound
-from tastypie.fields import *
-from tastypie.resources import ModelResource
+from tastypie.fields import NOT_PROVIDED, ApiField, BooleanField, CharField,\
+    DateField, DateTimeField, DecimalField, DictField, FileField, FloatField,\
+    IntegerField, ListField, TimeField, ToOneField, ToManyField
+from tastypie.resources import ALL, ModelResource
+from tastypie.utils import aware_datetime
+
 from core.models import Note, Subject, MediaBit
 from core.tests.mocks import MockRequest
 
-from tastypie.utils import aware_datetime, aware_date
-
 
 class ApiFieldTestCase(TestCase):
     fixtures = ['note_testdata.json']
@@ -23,7 +28,6 @@ class ApiFieldTestCase(TestCase):
         self.assertEqual(field_1.attribute, None)
         self.assertEqual(field_1._default, NOT_PROVIDED)
         self.assertEqual(field_1.null, False)
-        self.assertEqual(field_1.value, None)
         self.assertEqual(field_1.help_text, '')
         self.assertEqual(field_1.use_in, 'all')
 
@@ -32,7 +36,6 @@ class ApiFieldTestCase(TestCase):
         self.assertEqual(field_2.attribute, 'foo')
         self.assertEqual(field_2._default, True)
         self.assertEqual(field_2.null, True)
-        self.assertEqual(field_2.value, None)
         self.assertEqual(field_2.readonly, True)
         self.assertEqual(field_2.help_text, 'Foo.')
         self.assertEqual(field_1.use_in, 'all')
@@ -43,7 +46,8 @@ class ApiFieldTestCase(TestCase):
         field_4 = ApiField(use_in="detail")
         self.assertEqual(field_4.use_in, 'detail')
 
-        use_in_callable = lambda x: True
+        def use_in_callable(x):
+            return True
         field_5 = ApiField(use_in=use_in_callable)
         self.assertTrue(field_5.use_in is use_in_callable)
 
@@ -308,6 +312,9 @@ class DecimalFieldTestCase(TestCase):
         field_2 = DecimalField(default='18.5')
         self.assertEqual(field_2.dehydrate(bundle), Decimal('18.5'))
 
+        field_3 = DecimalField(default=21.5)
+        self.assertEqual(field_3.dehydrate(bundle), Decimal('21.5'))
+
     def test_hydrate(self):
         bundle = Bundle(data={
             'decimal-y': '18.50',
@@ -319,6 +326,16 @@ class DecimalFieldTestCase(TestCase):
         field_2 = DecimalField(default='18.5')
         self.assertEqual(field_2.hydrate(bundle), Decimal('18.5'))
 
+        bundle = Bundle(data={'foo': '1.5'})
+        field_3 = DecimalField()
+        field_3.instance_name = 'foo'
+        self.assertEqual(field_3.hydrate(bundle), Decimal('1.5'))
+
+        bundle = Bundle(data={'foo': 'xxx'})
+        field_4 = DecimalField(attribute='foo')
+        field_4.instance_name = 'foo'
+        self.assertRaises(ApiFieldError, field_4.hydrate, bundle)
+
     def test_model_resource_correct_association(self):
         api_field = ModelResource.api_field_from_django_field(models.DecimalField())
         self.assertEqual(api_field, DecimalField)
@@ -489,8 +506,6 @@ class DateFieldTestCase(TestCase):
         self.assertEqual(field_3.dehydrate(bundle), datetime.date(2010, 4, 2))
 
     def test_hydrate(self):
-        note = Note.objects.get(pk=1)
-
         bundle_1 = Bundle(data={
             'date': '2010-05-12',
         })
@@ -524,6 +539,10 @@ class DateFieldTestCase(TestCase):
         field_5.instance_name = 'date'
         self.assertEqual(field_5.hydrate(bundle_5), None)
 
+    def test_model_resource_correct_association(self):
+        api_field = ModelResource.api_field_from_django_field(models.DateField())
+        self.assertEqual(api_field, DateField)
+
 
 class DateTimeFieldTestCase(TestCase):
     fixtures = ['note_testdata.json']
@@ -554,8 +573,6 @@ class DateTimeFieldTestCase(TestCase):
         self.assertEqual(field_3.dehydrate(bundle), aware_datetime(2010, 4, 2, 1, 11))
 
     def test_hydrate(self):
-        note = Note.objects.get(pk=1)
-
         bundle_1 = Bundle(data={
             'datetime': '2010-05-12 10:36:28',
         })
@@ -592,11 +609,19 @@ class DateTimeFieldTestCase(TestCase):
         field_6.instance_name = 'datetime'
         self.assertRaises(ApiFieldError, field_6.hydrate, bundle_6)
 
+    def test_model_resource_correct_association(self):
+        api_field = ModelResource.api_field_from_django_field(models.DateTimeField())
+        self.assertEqual(api_field, DateTimeField)
+
 
 class UserResource(ModelResource):
     class Meta:
         resource_name = 'users'
         queryset = User.objects.all()
+        filtering = {
+            'id': ALL,
+            'username': ALL,
+        }
 
     def get_resource_uri(self, bundle_or_obj=None, url_name='api_dispatch_list'):
         if bundle_or_obj is None:
@@ -657,7 +682,8 @@ class ToOneFieldTestCase(TestCase):
         field_6 = ToOneField(UserResource, 'author', default=1, null=True, readonly=True, help_text="Points to a User.", use_in="detail")
         self.assertEqual(field_6.use_in, 'detail')
 
-        use_in_callable = lambda x: True
+        def use_in_callable(x):
+            return True
         field_7 = ToOneField(UserResource, 'author', default=1, null=True, readonly=True, help_text="Points to a User.", use_in=use_in_callable)
         self.assertTrue(field_7.use_in is use_in_callable)
 
@@ -716,17 +742,20 @@ class ToOneFieldTestCase(TestCase):
         field_2 = ToManyField(UserResource, lambda bundle: User.objects.filter(pk=1))
         self.assertEqual(field_2.dehydrate(bundle), ['/api/v1/users/1/'])
 
+        field_3 = ToOneField(UserResource, lambda bundle: None)
+        self.assertRaises(ApiFieldError, field_3.dehydrate, bundle)
+
     def test_dehydrate_full_detail_list(self):
         note = Note.objects.get(pk=1)
         request = MockRequest()
         bundle = Bundle(obj=note, request=request)
 
-        #details path with full_list=False
+        # details path with full_list=False
         request.path = "/api/v1/notes/"
         field_1 = ToOneField(UserResource, 'author', full=True, full_list=False)
         self.assertEqual(field_1.dehydrate(bundle), '/api/v1/users/1/')
 
-        #list path with full_detail=False
+        # list path with full_detail=False
         request.path = "/api/v1/notes/1/"
         field_1 = ToOneField(UserResource, 'author', full=True, full_detail=False)
         self.assertEqual(field_1.dehydrate(bundle, for_list=False), '/api/v1/users/1/')
@@ -811,12 +840,6 @@ class ToOneFieldTestCase(TestCase):
         fk_bundle = field_8.hydrate(bundle)
         self.assertEqual(field_8.hydrate(bundle), None)
         # Then use an unsaved object in the bundle also with ``null=True``.
-        new_note = Note(
-            title='Biplanes for all!',
-            slug='biplanes-for-all',
-            content='Somewhere, east of Manhattan, will lie the mythical land of planes with more one wing...'
-        )
-        new_bundle = Bundle(obj=new_note)
         field_9 = ToOneField(UserResource, 'author', null=True)
         field_9.instance_name = 'author'
         self.assertEqual(field_9.hydrate(bundle), None)
@@ -896,7 +919,8 @@ class ToOneFieldTestCase(TestCase):
 
     def test_traversed_attribute_dehydrate(self):
         user = User.objects.get(pk=1)
-        mediabit = MediaBit(note=Note(author=user))
+        note = Note.objects.create(author=user)
+        mediabit = MediaBit(note=note)
         bundle = Bundle(obj=mediabit)
 
         field_1 = ToOneField(UserResource, 'note__author')
@@ -934,7 +958,6 @@ class MediaBitResource(ModelResource):
 
 class ToManyFieldTestCase(TestCase):
     fixtures = ['note_testdata.json']
-    urls = 'core.tests.field_urls'
 
     def setUp(self):
         self.note_1 = Note.objects.get(pk=1)
@@ -1008,7 +1031,8 @@ class ToManyFieldTestCase(TestCase):
         field_6 = ToManyField(SubjectResource, 'author', default=1, null=True, readonly=True, help_text="Points to a User.", use_in="detail")
         self.assertEqual(field_6.use_in, 'detail')
 
-        use_in_callable = lambda x: True
+        def use_in_callable(x):
+            return True
         field_7 = ToManyField(SubjectResource, 'author', default=1, null=True, readonly=True, help_text="Points to a User.", use_in=use_in_callable)
         self.assertTrue(field_7.use_in is use_in_callable)
 
@@ -1039,12 +1063,8 @@ class ToManyFieldTestCase(TestCase):
         field_1 = ToManyField(SubjectResource, 'subjects')
         field_1.instance_name = 'm2m'
 
-        try:
-            # self.assertRaises isn't cooperating here. Do it the hard way.
+        with self.assertRaises(ApiFieldError):
             field_1.dehydrate(bundle_1)
-            self.fail()
-        except ApiFieldError:
-            pass
 
         field_2 = ToManyField(SubjectResource, 'subjects', null=True)
         field_2.instance_name = 'm2m'
@@ -1083,18 +1103,17 @@ class ToManyFieldTestCase(TestCase):
         bundle_6 = Bundle(obj=self.note_3)
         self.assertEqual(field_6.dehydrate(bundle_6), [])
 
-        try:
-            # Regression for missing variable initialization.
-            field_7 = ToManyField(SubjectResource, None)
-            field_7.instance_name = 'm2m'
-            bundle_7 = Bundle(obj=self.note_3)
+        # Regression for missing variable initialization.
+        field_7 = ToManyField(SubjectResource, None)
+        field_7.instance_name = 'm2m'
+        bundle_7 = Bundle(obj=self.note_3)
+
+        with self.assertRaises(ApiFieldError):
+            # ToManyField requires an attribute of some type.
             field_7.dehydrate(bundle_7)
-            self.fail('ToManyField requires an attribute of some type.')
-        except ApiFieldError:
-            pass
 
     def test_dehydrate_full_detail_list(self):
-        #details path with full_detail=False
+        # details path with full_detail=False
         field_1 = ToManyField(SubjectResource, 'subjects', full=True, full_detail=False)
         field_1.instance_name = 'm2m'
         request = MockRequest()
@@ -1102,7 +1121,7 @@ class ToManyFieldTestCase(TestCase):
         bundle_1 = Bundle(obj=self.note_1, request=request)
         self.assertEqual(field_1.dehydrate(bundle_1, for_list=False), ['/api/v1/subjects/1/', '/api/v1/subjects/2/'])
 
-        #list path with full_detail=False
+        # list path with full_detail=False
         field_2 = ToManyField(SubjectResource, 'subjects', full=True, full_detail=False)
         field_2.instance_name = 'm2m'
         request = MockRequest()
@@ -1121,7 +1140,7 @@ class ToManyFieldTestCase(TestCase):
         self.assertEqual(subject_bundle_list[1].obj.name, u'Photos')
         self.assertEqual(subject_bundle_list[1].obj.url, u'/photos/')
 
-        #list path with full_list=False
+        # list path with full_list=False
         field_3 = ToManyField(SubjectResource, 'subjects', full=True, full_list=False)
         field_3.instance_name = 'm2m'
         request = MockRequest()
@@ -1129,7 +1148,7 @@ class ToManyFieldTestCase(TestCase):
         bundle_3 = Bundle(obj=self.note_1, request=request)
         self.assertEqual(field_3.dehydrate(bundle_3), ['/api/v1/subjects/1/', '/api/v1/subjects/2/'])
 
-        #detail path with full_list=False
+        # detail path with full_list=False
         field_4 = ToManyField(SubjectResource, 'subjects', full=True, full_list=False)
         field_4.instance_name = 'm2m'
         request = MockRequest()
@@ -1148,7 +1167,7 @@ class ToManyFieldTestCase(TestCase):
         self.assertEqual(subject_bundle_list[1].obj.name, u'Photos')
         self.assertEqual(subject_bundle_list[1].obj.url, u'/photos/')
 
-        #list url with callable returning True
+        # list url with callable returning True
         field_5 = ToManyField(SubjectResource, 'subjects', full=True, full_list=lambda x: True)
         field_5.instance_name = 'm2m'
         request = MockRequest()
@@ -1167,7 +1186,7 @@ class ToManyFieldTestCase(TestCase):
         self.assertEqual(subject_bundle_list[1].obj.name, u'Photos')
         self.assertEqual(subject_bundle_list[1].obj.url, u'/photos/')
 
-        #list url with callable returning False
+        # list url with callable returning False
         field_6 = ToManyField(SubjectResource, 'subjects', full=True, full_list=lambda x: False)
         field_6.instance_name = 'm2m'
         request = MockRequest()
@@ -1175,7 +1194,7 @@ class ToManyFieldTestCase(TestCase):
         bundle_6 = Bundle(obj=self.note_1, request=request)
         self.assertEqual(field_6.dehydrate(bundle_6), ['/api/v1/subjects/1/', '/api/v1/subjects/2/'])
 
-        #detail url with callable returning True
+        # detail url with callable returning True
         field_7 = ToManyField(SubjectResource, 'subjects', full=True, full_detail=lambda x: True)
         field_7.instance_name = 'm2m'
         request = MockRequest()
@@ -1194,7 +1213,7 @@ class ToManyFieldTestCase(TestCase):
         self.assertEqual(subject_bundle_list[1].obj.name, u'Photos')
         self.assertEqual(subject_bundle_list[1].obj.url, u'/photos/')
 
-        #detail url with callable returning False
+        # detail url with callable returning False
         field_8 = ToManyField(SubjectResource, 'subjects', full=True, full_detail=lambda x: False)
         field_8.instance_name = 'm2m'
         request = MockRequest()
@@ -1202,7 +1221,7 @@ class ToManyFieldTestCase(TestCase):
         bundle_8 = Bundle(obj=self.note_1, request=request)
         self.assertEqual(field_8.dehydrate(bundle_8, for_list=False), ['/api/v1/subjects/1/', '/api/v1/subjects/2/'])
 
-        #detail url with full_detail=True and get parameters
+        # detail url with full_detail=True and get parameters
         field_9 = ToManyField(SubjectResource, 'subjects', full=True, full_detail=True)
         field_9.instance_name = 'm2m'
         request = HttpRequest()
@@ -1225,7 +1244,6 @@ class ToManyFieldTestCase(TestCase):
         self.assertEqual(subject_bundle_list[1].obj.url, u'/photos/')
 
     def test_dehydrate_with_callable(self):
-        note = Note()
         bundle_1 = Bundle(obj=self.note_2)
         field_1 = ToManyField(SubjectResource, attribute=lambda bundle: Subject.objects.filter(notes=bundle.obj, name__startswith='Personal'))
         field_1.instance_name = 'm2m'
