@@ -1,5 +1,4 @@
-from settings import *
+from settings import *  # flake8: noqa
 INSTALLED_APPS.append('alphanumeric')
 
 ROOT_URLCONF = 'alphanumeric.urls'
-
