@@ -1,6 +1,5 @@
-from settings import *
+from settings import *  # flake8: noqa
 INSTALLED_APPS.append('django.contrib.sessions')
 INSTALLED_APPS.append('basic')
 
 ROOT_URLCONF = 'basic.urls'
-
