@@ -1,7 +1,12 @@
-from settings import *
+from settings import *  # flake8: noqa
 INSTALLED_APPS.append('django.contrib.sessions')
 INSTALLED_APPS.append('core')
-INSTALLED_APPS.append('oauth_provider')
+
+try:
+    import oauth_provider  # flake8: noqa
+    INSTALLED_APPS.append('oauth_provider')
+except ImportError:
+    pass
 
 ROOT_URLCONF = 'core.tests.api_urls'
 MEDIA_URL = 'http://localhost:8080/media/'
@@ -23,4 +28,3 @@ LOGGING = {
         },
     }
 }
-
